import cPickle

def srand( ULI seed ):
  c_srand( seed )

####################################################################
# Memory managment low-level functions
####################################################################

cdef ULI size( ULI pointers, ULI ulis, ULI usis, ULI ints, ULI doubles ):
  """An auxiliary function."""
  return c_size_pointer * pointers + c_size_ULI * ulis + \
         c_size_USI * usis + c_size_int * ints + \
         c_size_double * doubles;

cdef SInd* new_SInd( ULI num, ULI chrom_size ):
  """The SInd structure constructor."""
  cdef SInd *p
  cdef ULI i
  
    # the main structure
  p = <SInd*> c_new( num, size( 2, 0, 3, 0, 3 ) )
  
    # the chromosome
  for i from 0 <= i < num:
    p[i].puchrom = c_new_ULI( chrom_size )
    p[i].pfen = NULL
  
    # the fenotype should be created by a problem-specific function
  
  return p

cdef void free_SInd( SInd *p, ULI num ):
  """The SInd structure destructor."""
  cdef ULI i
  
    # free the chromosome
  for i from 0 <= i < num:
    c_free( <void*> p[i].puchrom )
  
    # the fenotype should be freed by a problem-specific function
  
    # free the main structure
  c_free( <void*> p )

cdef SPop* new_SPop( ULI num, ULI ind_num, ULI chrom_size ):
  """The constructor of the SPop structure(s)."""
  cdef SPop *p
  cdef ULI i
  
  p = <SPop*> c_new( num, size( 1, 7, 0, 2, 6 ) )
  for i from 0 <= i < num:
    p[i].ind = new_SInd( ind_num, chrom_size )
  return p

cdef void free_SPop( SPop *p, ULI pop_num, ULI ind_num ):
  """The destructor of the SPop structure(s)."""
  cdef ULI i
  
  for i from 0 <= i <= pop_num:
    free_SInd( p[i].ind, ind_num )
  
  c_free( p )

cdef class Pop:
  
  cdef SPop *p
  cdef readonly ULI pops, inds, chroms
  
  def __init__( self, ULI pops, ULI inds, ULI chroms ):
    self.p = new_SPop( pops, inds, chroms )
    self.pops = pops
    self.inds = inds
    self.chroms = chroms
  
  def __del__( self ):
    free_SPop( self.p, self.pops, self.inds )
  
  def show( self ):
    cdef ULI p,i,c
    
    for p from 0 <= p < self.pops:
      for i from 0 <= i < self.inds:
        for c from 0 <= c < self.chroms:
          print 'Population', p, 'individual', i, 'chromosome', c
          pi = self.p[p].ind[i].puchrom[c]
          print 'Read', pi
          print 'Write',
          self.p[p].ind[i].puchrom[c] = 10
          pi = self.p[p].ind[i].puchrom[c]
          print pi

def test():
  t = Pop( 3, 70, 50 )
  t.show()

def test2():
  t = Pop( 3, 20, 50 )
  t.show()
